#include <iostream>
#include <string>
#include <vector>
#include <stdio.h>
#include <cstdlib>

#include "appl_grid/appl_grid.h"
#include "appl_grid/Directory.h"


#include "appl_grid/appl_timer.h"

#include <TCanvas.h>
#include <TH1D.h>
#include <TFile.h>
#include <TPad.h>

#include "hoppet_v1.h"


// lhapdf routines
#include "LHAPDF/LHAPDF.h"
extern "C" void evolvepdf_(const double& x, const double& Q, double* xf); 
extern "C" double alphaspdf_(const double& Q);


#define DBG false


static const int nFlavours = 5;



void binwidth(TH1D* h) { 
  for ( int i=1 ; i<=h->GetNbinsX() ; i++ ) {
    double dx = h->GetBinLowEdge(i+1) - h->GetBinLowEdge(i);
    h->SetBinContent( i,  h->GetBinContent(i)/dx );
    h->SetBinError( i,  h->GetBinError(i)/dx );
  }
}


double getRealMinimum( TH1D* h ) { 
  double _min = 0;
  bool set = false;
  for ( int i=1 ; i<=h->GetNbinsX() ; i++ ) {
    double y = h->GetBinContent(i);
    if ( y!=0 ) { 
      if ( set ) { 
	if ( y<_min ) _min = y;
      }
      else _min = y;
    }
  }

  return _min;
}


double getRealMaximum( TH1D* h ) { 
  double _max = 0;
  bool set = false;
  for ( int i=1 ; i<=h->GetNbinsX() ; i++ ) {
    double y = h->GetBinContent(i);
    if ( y!=0 ) { 
      if ( set ) { 
	if ( y<_max ) _max = y;
      }
      else _max = y;
    }
  }

  return _max;
}


TH1D* divide( const TH1D* h1, const TH1D* h2 ) {

  if ( h1==NULL || h2==NULL ) return NULL;
 
  TH1D* h = (TH1D*)h1->Clone();

  if ( DBG ) std::cout << "histograms " << h1->GetTitle() << " " << h2->GetTitle() << std::endl;

  
  for ( int i=1 ; i<=h1->GetNbinsX() ; i++ ) { 
    double b  = h2->GetBinContent(i);
    //    double be = h2->GetBinError(i);
    double t  = h1->GetBinContent(i);
    //  double te = h1->GetBinError(i);

    double r  = ( b!=0 ? t/b : 0 );
    //    double re = ( b!=0 ? sqrt((r+1)*r/b) : 0 );
    double re = 0;
    
    h->SetBinContent( i, r );
    h->SetBinError( i, re ) ;

    //    if ( DBG ) std::cout << "\tx=" << h->GetBinCenter(i) << "\tratio=" << r << std::endl;
  } 

  double hmin = h->GetBinContent(1);
  double hmax = h->GetBinContent(1);
  
  for ( int i=2 ; i<=h->GetNbinsX() ; i++ ) { 
    double d = h->GetBinContent(i);
    if ( hmin>d ) hmin=d;
    if ( hmax<d ) hmax=d; 
  }

  if ( DBG ) std::cout << "\tmin ratio = " << hmin << "\tmax ratio = " << hmax << std::endl;

  double _max = getRealMaximum( h );
  double _min = getRealMinimum( h );

  if ( _max<1.1 ) h->SetMaximum(1.1);
  if ( _min>0.9 ) h->SetMinimum(0.9);

  if ( _max<1.01 ) h->SetMaximum(1.01);
  if ( _min>0.99 ) h->SetMinimum(0.99);

  if ( _max<1.001 ) h->SetMaximum(1.001);
  if ( _min>0.999 ) h->SetMinimum(0.999);

  if ( _max<1.0001 ) h->SetMaximum(1.0001);
  if ( _min>0.9999 ) h->SetMinimum(0.9999);


  return h;
}


void GetPdf(const double& x, const double& Q, double* xf) { 
  evolvepdf_( x, Q, xf);    
  //hoppeteval_( x, Q, xf);    
  return; 
}



int main(int argc, char** argv) { 

  if ( argc<2 ) return -1;

  std::string gridName(argv[1]);
  
  appl::grid g( gridName.c_str() );

  //  g.untrim();
  int gusize = g.size();

  bool istrimmed = g.isTrimmed();

  g.trim(); 

  int gtsize = g.size();

  if ( gtsize != gusize ) std::cout << "gridsize : " 
				    << "untrimmed " << gusize 
				    << "\ttrimmed " << gtsize 
				    << "  (" << (gtsize*100./gusize) << "%)\tistrimmed " << istrimmed << std::endl;


  // get all the reference histograms

  TFile* f;
  f = new TFile(argv[1]);

  std::string outFileName = gridName.substr(0,gridName.find(".root"));
  outFileName += "_standSimple.root";

  TFile* fout = new TFile(outFileName.c_str(), "recreate");


  Directory ref("reference");
  ref.push();

  TH1D* reference = (TH1D*)f->Get("grid/reference");

  //  binwidth( reference );

  reference->Write();
  ref.pop();


  
  // now calculate all the cross sections

  std::string pdfpath = std::string(PDFPATH) + "/";
  
  

  std::string _pdfname = pdfpath + "CT10.LHgrid";  
  // std::string _pdfname = pdfpath + "CT10nlo.LHgrid";  
  // std::string _pdfname = pdfpath + "cteq66.LHgrid";  
  int Npdf = 0;


  if ( argc>2 ) Npdf = std::atoi(argv[2]);
  if ( argc>3 ) _pdfname = pdfpath + argv[3];

  
  std::cout << "trying to setup pdf set >" << _pdfname << "<" << " " << Npdf << std::endl; 

  

  // setup pdf

  LHAPDF::initPDFSet(_pdfname.c_str(), Npdf );

  std::cout << "setup pdf" << std::endl; 



  Directory xsDir("xSection");
  xsDir.push();

  const int nLoops = g.nloops();

  std::cout << "performing convolution ..." << std::endl; 

  struct timeval mytimer = appl_timer_start();

  TH1D* xsec = g.convolute(evolvepdf_, alphaspdf_, nLoops); 

  double mytime = appl_timer_stop(mytimer);

  std::cout << "done (" << mytime << " ms)" << std::endl; 
  
  xsec->SetName("xsec");
  xsec->SetTitle(reference->GetTitle());

  xsec->Write();  

  xsec->SetLineColor(kBlue);
  xsec->DrawCopy();
  reference->SetLineColor(kBlack);
  reference->SetLineStyle(2);
  reference->DrawCopy("same");

  gPad->Print("xsec.pdf");

  xsDir.pop();



  // now take all the ratios etc

  Directory ratiodir("ratio");
  ratiodir.push();

  TH1D* ratio = divide( xsec, reference ); 
  if ( ratio ) {
    ratio->SetName("ratio");
    ratio->Write();
  }
  
  ratio->DrawCopy();
  gPad->Print("ratio.pdf");
  gPad->Print("ratio.png");


  ratiodir.pop();

  fout->Write();
  fout->Close();

  std::cout << "individual components..." << std::endl;

  TH1D* hLOxsec   = g.convolute(evolvepdf_, alphaspdf_, 0); 
  TH1D* hNLOxsec  = 0;
  TH1D* hNLOOxsec = 0;
  
  if ( g.nloops()>0 ) { 
    hNLOxsec  = g.convolute(evolvepdf_, alphaspdf_,  1); 
    hNLOOxsec = g.convolute(evolvepdf_, alphaspdf_, -1); 
  }


  hLOxsec->DrawCopy();
  gPad->Print("LO.pdf");

  if ( hNLOxsec ) {
    hNLOxsec->DrawCopy();
    gPad->Print("NLO.pdf");
  }    

  if ( hNLOOxsec ) {
    hNLOOxsec->DrawCopy();
    gPad->Print("NLO-only.pdf");
  }


  //   int nloops = g.nloops();

  //   std::cout << "nloops: " << nloops << std::endl;
  
  //   std::vector< std::vector<double> > vxsec(nloops);      /// cross section up to each order
  //   std::vector< std::vector<double> > vxsec_only(nloops); /// cross section order terms only

  //   for ( int i=0 ; i<nloops ; i++ ) { 
  //     vxsec[i]                 = g.vconvolute(evolvepdf_, alphaspdf_,  i); 
  //     if ( i>0 ) vxsec_only[i] = g.vconvolute(evolvepdf_, alphaspdf_, -i); 
  //   }

  
  //   for ( unsigned i=0 ; i<vxsec[0].size() ; i++ ) { 

  //     std::cout << i << " ref " << reference->GetBinContent(i+1) << "\txsec ";

  //     for ( unsigned j=0 ; j<vxsec.size() ; j++ )       std::cout << " " << vxsec[j][i];
  //     std::cout << "\tcontrib. ";
  //     for ( unsigned j=1 ; j<vxsec_only.size() ; j++ )  std::cout << " " << vxsec_only[j][i]; 
  //     std::cout << std::endl;
    
  //   }


  return 0;
}
